@@ -72,6 +72,9 @@
 #include "libds/common/roles.h"
 #include "lib/util/samba_util.h"
 #include "libcli/auth/ntlm_check.h"
+#include "lib/crypto/gnutls_helpers.h"
+#include "lib/util/smb_strtox.h"
+#include "auth/credentials/credentials.h"
 
 #ifdef HAVE_HTTPCONNECTENCRYPT
 #include <cups/http.h>
@@ -96,6 +99,19 @@ int lpcfg_rpc_high_port(struct loadparm_context *lp_ctx)
 	return lp_ctx->globals->rpc_high_port;
 }
 
+enum samba_weak_crypto lpcfg_weak_crypto(struct loadparm_context *lp_ctx)
+{
+	if (lp_ctx->globals->weak_crypto == SAMBA_WEAK_CRYPTO_UNKNOWN) {
+		lp_ctx->globals->weak_crypto = SAMBA_WEAK_CRYPTO_DISALLOWED;
+
+		if (samba_gnutls_weak_crypto_allowed()) {
+			lp_ctx->globals->weak_crypto = SAMBA_WEAK_CRYPTO_ALLOWED;
+		}
+	}
+
+	return lp_ctx->globals->weak_crypto;
+}
+
 /**
  * Convenience routine to grab string parameters into temporary memory
  * and run standard_sub_basic on them.
@@ -713,76 +729,6 @@ struct loadparm_service *lpcfg_add_service(struct loadparm_context *lp_ctx,
 	return lp_ctx->services[i];
 }
 
-/**
- * Add a new home service, with the specified home directory, defaults coming
- * from service ifrom.
- */
-
-bool lpcfg_add_home(struct loadparm_context *lp_ctx,
-		 const char *pszHomename,
-		 struct loadparm_service *default_service,
-		 const char *user, const char *pszHomedir)
-{
-	struct loadparm_service *service;
-
-	service = lpcfg_add_service(lp_ctx, default_service, pszHomename);
-
-	if (service == NULL)
-		return false;
-
-	if (!(*(default_service->path))
-	    || strequal(default_service->path, lp_ctx->sDefault->path)) {
-		service->path = talloc_strdup(service, pszHomedir);
-	} else {
-		service->path = string_sub_talloc(service, lpcfg_path(default_service, lp_ctx->sDefault, service), "%H", pszHomedir);
-	}
-
-	if (!(*(service->comment))) {
-		service->comment = talloc_asprintf(service, "Home directory of %s", user);
-	}
-	service->available = default_service->available;
-	service->browseable = default_service->browseable;
-
-	DEBUG(3, ("adding home's share [%s] for user '%s' at '%s'\n",
-		  pszHomename, user, service->path));
-
-	return true;
-}
-
-/**
- * Add a new printer service, with defaults coming from service iFrom.
- */
-
-bool lpcfg_add_printer(struct loadparm_context *lp_ctx,
-		       const char *pszPrintername,
-		       struct loadparm_service *default_service)
-{
-	const char *comment = "From Printcap";
-	struct loadparm_service *service;
-	service = lpcfg_add_service(lp_ctx, default_service, pszPrintername);
-
-	if (service == NULL)
-		return false;
-
-	/* note that we do NOT default the availability flag to True - */
-	/* we take it from the default service passed. This allows all */
-	/* dynamic printers to be disabled by disabling the [printers] */
-	/* entry (if/when the 'available' keyword is implemented!).    */
-
-	/* the printer name is set to the service name. */
-	lpcfg_string_set(service, &service->_printername, pszPrintername);
-	lpcfg_string_set(service, &service->comment, comment);
-	service->browseable = default_service->browseable;
-	/* Printers cannot be read_only. */
-	service->read_only = false;
-	/* Printer services must be printable. */
-	service->printable = true;
-
-	DEBUG(3, ("adding printer service %s\n", pszPrintername));
-
-	return true;
-}
-
 /**
  * Map a parameter's string representation to something we can use.
  * Returns False if the parameter string is not recognised, else TRUE.
@@ -854,6 +800,16 @@ bool lpcfg_parm_is_cmdline(struct loadparm_context *lp_ctx, const char *name)
 	return lp_ctx->flags[parmnum] & FLAG_CMDLINE;
 }
 
+bool lpcfg_parm_is_unspecified(struct loadparm_context *lp_ctx, const char *name)
+{
+	int parmnum;
+
+	parmnum = lpcfg_map_parameter(name);
+	if (parmnum == -1) return false;
+
+	return lp_ctx->flags[parmnum] & FLAG_DEFAULT;
+}
+
 /**
  * Find a service by name. Otherwise works like get_service.
  */
@@ -1087,39 +1043,6 @@ fail:
 
 }
 
-/*******************************************************************
- Check if a config file has changed date.
-********************************************************************/
-bool lpcfg_file_list_changed(struct loadparm_context *lp_ctx)
-{
-	struct file_lists *f;
-	DEBUG(6, ("lpcfg_file_list_changed()\n"));
-
-	for (f = lp_ctx->file_lists; f != NULL; f = f->next) {
-		char *n2;
-		time_t mod_time;
-
-		n2 = standard_sub_basic(lp_ctx, f->name);
-
-		DEBUGADD(6, ("file %s -> %s  last mod_time: %s\n",
-			     f->name, n2, ctime(&f->modtime)));
-
-		mod_time = file_modtime(n2);
-
-		if (mod_time && ((f->modtime != mod_time) || (f->subfname == NULL) || (strcmp(n2, f->subfname) != 0))) {
-			DEBUGADD(6, ("file %s modified: %s\n", n2,
-				  ctime(&mod_time)));
-			f->modtime = mod_time;
-			talloc_free(f->subfname);
-			f->subfname = talloc_strdup(f, n2);
-			TALLOC_FREE(n2);
-			return true;
-		}
-		TALLOC_FREE(n2);
-	}
-	return false;
-}
-
 /*
  * set the value for a P_ENUM
  */
@@ -1378,19 +1301,6 @@ bool handle_ldap_debug_level(struct loadparm_context *lp_ctx, struct loadparm_se
 	return true;
 }
 
-bool handle_netbios_aliases(struct loadparm_context *lp_ctx, struct loadparm_service *service,
-			    const char *pszParmValue, char **ptr)
-{
-	TALLOC_FREE(lp_ctx->globals->netbios_aliases);
-	lp_ctx->globals->netbios_aliases = str_list_make_v3_const(lp_ctx->globals->ctx,
-								  pszParmValue, NULL);
-
-	if (lp_ctx->s3_fns) {
-		return lp_ctx->s3_fns->set_netbios_aliases(lp_ctx->globals->netbios_aliases);
-	}
-	return true;
-}
-
 /*
  * idmap related parameters
  */
@@ -1849,8 +1759,15 @@ bool lpcfg_do_global_parameter(struct loadparm_context *lp_ctx,
 	}
 
 	if (parm_table[parmnum].flags & FLAG_DEPRECATED) {
-		DEBUG(1, ("WARNING: The \"%s\" option is deprecated\n",
-			  pszParmName));
+		char *suppress_env = getenv("SAMBA_DEPRECATED_SUPPRESS");
+		bool print_warning = (suppress_env == NULL
+				      || suppress_env[0] == '\0');
+		if (print_warning) {
+			DBG_WARNING("WARNING: The \"%s\" option "
+				    "is deprecated\n",
+				    pszParmName);
+
+		}
 	}
 
 	parm_ptr = lpcfg_parm_ptr(lp_ctx, NULL, &parm_table[parmnum]);
@@ -1882,8 +1799,15 @@ bool lpcfg_do_service_parameter(struct loadparm_context *lp_ctx,
 	}
 
 	if (parm_table[parmnum].flags & FLAG_DEPRECATED) {
-		DEBUG(1, ("WARNING: The \"%s\" option is deprecated\n",
-			  pszParmName));
+		char *suppress_env = getenv("SAMBA_DEPRECATED_SUPPRESS");
+		bool print_warning = (suppress_env == NULL
+				      || suppress_env[0] == '\0');
+		if (print_warning) {
+			DBG_WARNING("WARNING: The \"%s\" option "
+				    "is deprecated\n",
+				    pszParmName);
+
+		}
 	}
 
 	if (parm_table[parmnum].p_class == P_GLOBAL) {
@@ -2379,11 +2303,14 @@ bool lpcfg_dump_a_parameter(struct loadparm_context *lp_ctx,
 				local_parm_name, parm_opt);
 			if (parm_opt_value) {
 				fprintf(f, "%s\n", parm_opt_value);
+				TALLOC_FREE(local_parm_name);
 				return true;
 			}
 		}
+		TALLOC_FREE(local_parm_name);
 		return false;
 	}
+	TALLOC_FREE(local_parm_name);
 
 	/* parameter is not parametric, search the table */
 	parm = lpcfg_parm_struct(lp_ctx, parm_name);
@@ -2533,31 +2460,6 @@ void init_printer_values(struct loadparm_context *lp_ctx, TALLOC_CTX *ctx,
 	}
 }
 
-/**
- * Unload unused services.
- */
-
-void lpcfg_killunused(struct loadparm_context *lp_ctx,
-		   struct smbsrv_connection *smb,
-		   bool (*snumused) (struct smbsrv_connection *, int))
-{
-	int i;
-
-	if (lp_ctx->s3_fns != NULL) {
-		smb_panic("Cannot be used from an s3 loadparm ctx");
-	}
-
-	for (i = 0; i < lp_ctx->iNumServices; i++) {
-		if (lp_ctx->services[i] == NULL)
-			continue;
-
-		if (!snumused || !snumused(smb, i)) {
-			talloc_free(lp_ctx->services[i]);
-			lp_ctx->services[i] = NULL;
-		}
-	}
-}
-
 
 static int lpcfg_destructor(struct loadparm_context *lp_ctx)
 {
@@ -2607,6 +2509,7 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 	lp_ctx->globals->ctx = lp_ctx->globals;
 	lp_ctx->globals->rpc_low_port = SERVER_TCP_LOW_PORT;
 	lp_ctx->globals->rpc_high_port = SERVER_TCP_HIGH_PORT;
+	lp_ctx->globals->weak_crypto = SAMBA_WEAK_CRYPTO_UNKNOWN;
 	lp_ctx->sDefault = talloc_zero(lp_ctx, struct loadparm_service);
 	lp_ctx->flags = talloc_zero_array(lp_ctx, unsigned int, num_parameters());
 
@@ -2656,12 +2559,11 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 	lpcfg_do_global_parameter(lp_ctx, "debug timestamp", "Yes");
 	lpcfg_do_global_parameter(lp_ctx, "debug prefix timestamp", "No");
 	lpcfg_do_global_parameter(lp_ctx, "debug hires timestamp", "Yes");
+	lpcfg_do_global_parameter(lp_ctx, "debug syslog format", "No");
 	lpcfg_do_global_parameter(lp_ctx, "debug pid", "No");
 	lpcfg_do_global_parameter(lp_ctx, "debug uid", "No");
 	lpcfg_do_global_parameter(lp_ctx, "debug class", "No");
 
-	lpcfg_do_global_parameter(lp_ctx, "share backend", "classic");
-
 	lpcfg_do_global_parameter(lp_ctx, "server role", "auto");
 	lpcfg_do_global_parameter(lp_ctx, "domain logons", "No");
 	lpcfg_do_global_parameter(lp_ctx, "domain master", "Auto");
@@ -2762,7 +2664,7 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 
 	lpcfg_do_global_parameter(lp_ctx, "winbind separator", "\\");
 	lpcfg_do_global_parameter(lp_ctx, "winbind sealed pipes", "True");
-	lpcfg_do_global_parameter(lp_ctx, "winbind scan trusted domains", "True");
+	lpcfg_do_global_parameter(lp_ctx, "winbind scan trusted domains", "False");
 	lpcfg_do_global_parameter(lp_ctx, "require strong key", "True");
 	lpcfg_do_global_parameter(lp_ctx, "winbindd socket directory", dyn_WINBINDD_SOCKET_DIR);
 	lpcfg_do_global_parameter(lp_ctx, "ntp signd socket directory", dyn_NTP_SIGND_SOCKET_DIR);
@@ -2793,6 +2695,8 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 	lpcfg_do_global_parameter(lp_ctx, "krb5 port", "88");
 	lpcfg_do_global_parameter(lp_ctx, "kpasswd port", "464");
 
+	lpcfg_do_global_parameter(lp_ctx, "kdc enable fast", "True");
+
 	lpcfg_do_global_parameter(lp_ctx, "nt status support", "True");
 
 	lpcfg_do_global_parameter(lp_ctx, "max wins ttl", "518400"); /* 6 days */
@@ -2803,7 +2707,9 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 	lpcfg_do_global_parameter(lp_ctx, "tls keyfile", "tls/key.pem");
 	lpcfg_do_global_parameter(lp_ctx, "tls certfile", "tls/cert.pem");
 	lpcfg_do_global_parameter(lp_ctx, "tls cafile", "tls/ca.pem");
-	lpcfg_do_global_parameter(lp_ctx, "tls priority", "NORMAL:-VERS-SSL3.0");
+	lpcfg_do_global_parameter(lp_ctx,
+				  "tls priority",
+				  "NORMAL:-VERS-SSL3.0");
 
 	lpcfg_do_global_parameter(lp_ctx, "nsupdate command", "/usr/bin/nsupdate -g");
 
@@ -2905,7 +2811,7 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 
 	lpcfg_do_global_parameter(lp_ctx, "ldap page size", "1000");
 
-	lpcfg_do_global_parameter(lp_ctx, "kernel share modes", "yes");
+	lpcfg_do_global_parameter(lp_ctx, "kernel share modes", "no");
 
 	lpcfg_do_global_parameter(lp_ctx, "strict locking", "Auto");
 
@@ -3021,6 +2927,8 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 
 	lpcfg_do_global_parameter(lp_ctx, "smb2 leases", "yes");
 
+	lpcfg_do_global_parameter(lp_ctx, "server multi channel support", "yes");
+
 	lpcfg_do_global_parameter(lp_ctx, "kerberos encryption types", "all");
 
 	lpcfg_do_global_parameter(lp_ctx,
@@ -3048,6 +2956,55 @@ struct loadparm_context *loadparm_init(TALLOC_CTX *mem_ctx)
 	lpcfg_do_global_parameter(
 		lp_ctx, "ldap max search request size", "256000");
 
+	/* Async DNS query timeout in seconds. */
+	lpcfg_do_global_parameter(lp_ctx, "async dns timeout", "10");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "client smb encrypt",
+				  "default");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "client use kerberos",
+				  "desired");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "client protection",
+				  "default");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "smbd max xattr size",
+				  "65536");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "acl flag inherited canonicalization",
+				  "yes");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "winbind use krb5 enterprise principals",
+				  "yes");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "client smb3 signing algorithms",
+				  DEFAULT_SMB3_SIGNING_ALGORITHMS);
+	lpcfg_do_global_parameter(lp_ctx,
+				  "server smb3 signing algorithms",
+				  DEFAULT_SMB3_SIGNING_ALGORITHMS);
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "client smb3 encryption algorithms",
+				  DEFAULT_SMB3_ENCRYPTION_ALGORITHMS);
+	lpcfg_do_global_parameter(lp_ctx,
+				  "server smb3 encryption algorithms",
+				  DEFAULT_SMB3_ENCRYPTION_ALGORITHMS);
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "min domain uid",
+				  "1000");
+
+	lpcfg_do_global_parameter(lp_ctx,
+				  "rpc start on demand helpers",
+				  "yes");
+
 	for (i = 0; parm_table[i].label; i++) {
 		if (!(lp_ctx->flags[i] & FLAG_CMDLINE)) {
 			lp_ctx->flags[i] |= FLAG_DEFAULT;
@@ -3156,9 +3113,11 @@ static bool lpcfg_update(struct loadparm_context *lp_ctx)
 	settings.timestamp_logs = lp_ctx->globals->timestamp_logs;
 	settings.debug_prefix_timestamp = lp_ctx->globals->debug_prefix_timestamp;
 	settings.debug_hires_timestamp = lp_ctx->globals->debug_hires_timestamp;
+	settings.debug_syslog_format = lp_ctx->globals->debug_syslog_format;
 	settings.debug_pid = lp_ctx->globals->debug_pid;
 	settings.debug_uid = lp_ctx->globals->debug_uid;
 	settings.debug_class = lp_ctx->globals->debug_class;
+	settings.max_log_size = lp_ctx->globals->max_log_size;
 	debug_set_settings(&settings, lp_ctx->globals->logging,
 			   lp_ctx->globals->syslog,
 			   lp_ctx->globals->syslog_only);
@@ -3214,9 +3173,12 @@ static bool lpcfg_load_internal(struct loadparm_context *lp_ctx,
 	char *n2;
 	bool bRetval;
 
-	filename = talloc_strdup(lp_ctx, filename);
+	if (lp_ctx->szConfigFile != NULL) {
+		talloc_free(discard_const_p(char, lp_ctx->szConfigFile));
+		lp_ctx->szConfigFile = NULL;
+	}
 
-	lp_ctx->szConfigFile = filename;
+	lp_ctx->szConfigFile = talloc_strdup(lp_ctx, filename);
 
 	if (lp_ctx->s3_fns) {
 		return lp_ctx->s3_fns->load(filename);
@@ -3362,47 +3324,6 @@ const char *lpcfg_servicename(const struct loadparm_service *service)
 	return service ? lpcfg_string((const char *)service->szService) : NULL;
 }
 
-/**
- * A useful volume label function.
- */
-const char *lpcfg_volume_label(struct loadparm_service *service, struct loadparm_service *sDefault)
-{
-	const char *ret;
-	ret = lpcfg_string((const char *)((service != NULL && service->volume != NULL) ?
-				       service->volume : sDefault->volume));
-	if (!*ret)
-		return lpcfg_servicename(service);
-	return ret;
-}
-
-/**
- * Return the correct printer name.
- */
-const char *lpcfg_printername(struct loadparm_service *service, struct loadparm_service *sDefault)
-{
-	const char *ret;
-	ret = lpcfg_string((const char *)((service != NULL && service->_printername != NULL) ?
-				       service->_printername : sDefault->_printername));
-	if (ret == NULL || (ret != NULL && *ret == '\0'))
-		ret = lpcfg_servicename(service);
-
-	return ret;
-}
-
-
-/**
- * Return the max print jobs per queue.
- */
-int lpcfg_maxprintjobs(struct loadparm_service *service, struct loadparm_service *sDefault)
-{
-	int maxjobs = lpcfg_max_print_jobs(service, sDefault);
-
-	if (maxjobs <= 0 || maxjobs >= PRINT_MAX_JOBID)
-		maxjobs = PRINT_MAX_JOBID - 1;
-
-	return maxjobs;
-}
-
 struct smb_iconv_handle *lpcfg_iconv_handle(struct loadparm_context *lp_ctx)
 {
 	if (lp_ctx == NULL) {
@@ -3521,6 +3442,15 @@ int lpcfg_client_ipc_signing(struct loadparm_context *lp_ctx)
 	return client_ipc_signing;
 }
 
+enum credentials_use_kerberos lpcfg_client_use_kerberos(struct loadparm_context *lp_ctx)
+{
+	if (lpcfg_weak_crypto(lp_ctx) == SAMBA_WEAK_CRYPTO_DISALLOWED) {
+		return CRED_USE_KERBEROS_REQUIRED;
+	}
+
+	return lpcfg__client_use_kerberos(lp_ctx);
+}
+
 bool lpcfg_server_signing_allowed(struct loadparm_context *lp_ctx, bool *mandatory)
 {
 	bool allowed = true;
@@ -3636,3 +3566,33 @@ char *lpcfg_substituted_string(TALLOC_CTX *mem_ctx,
 					     raw_value,
 					     lp_sub->private_data);
 }
+
+/**
+ * @brief Parse a string value of a given parameter to its integer enum value.
+ *
+ * @param[in]  param_name    The parameter name (e.g. 'client smb encrypt')
+ *
+ * @param[in]  param_value   The parameter value (e.g. 'required').
+ *
+ * @return The integer value of the enum the param_value matches or INT32_MIN
+ * on error.
+ */
+int32_t lpcfg_parse_enum_vals(const char *param_name,
+			      const char *param_value)
+{
+	struct parm_struct *parm = NULL;
+	int32_t ret = INT32_MIN;
+	bool ok;
+
+	parm = lpcfg_parm_struct(NULL, param_name);
+	if (parm == NULL) {
+		return INT32_MIN;
+	}
+
+	ok = lp_set_enum_parm(parm, param_value, &ret);
+	if (!ok) {
+		return INT32_MIN;
+	}
+
+	return ret;
+}
