@@ -1208,10 +1208,23 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 
 		DBG_DEBUG("stream_name: %s\n", smb_fname_str_dbg(stream_name));
 
+		rc = vfs_stat(handle->conn, stream_name);
+		if (rc == -1 && errno != ENOENT) {
+			ok = false;
+			goto fail;
+		}
+
+		status = openat_pathref_fsp(handle->conn->cwd_fsp, stream_name);
+		if (!NT_STATUS_IS_OK(status) &&
+		    !NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND))
+		{
+			ok = false;
+			goto fail;
+		}
+
 		status = SMB_VFS_CREATE_FILE(
 			handle->conn,			/* conn */
 			NULL,				/* req */
-			&handle->conn->cwd_fsp,		/* dirfsp */
 			stream_name,			/* fname */
 			FILE_GENERIC_WRITE,		/* access_mask */
 			FILE_SHARE_READ | FILE_SHARE_WRITE, /* share_access */
@@ -1241,13 +1254,13 @@ static bool ad_convert_xattr(vfs_handle_struct *handle,
 		if (nwritten == -1) {
 			DBG_ERR("SMB_VFS_PWRITE failed\n");
 			saved_errno = errno;
-			close_file(NULL, fsp, ERROR_CLOSE);
+			close_file_free(NULL, &fsp, ERROR_CLOSE);
 			errno = saved_errno;
 			ok = false;
 			goto fail;
 		}
 
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
 			ok = false;
 			goto fail;
@@ -1294,6 +1307,7 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 	NTSTATUS status;
 	int saved_errno = 0;
 	int cmp;
+	int rc;
 
 	cmp = memcmp(ad->ad_filler, AD_FILLER_TAG_OSX, ADEDLEN_FILLER);
 	if (cmp != 0) {
@@ -1338,10 +1352,21 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 
 	DBG_DEBUG("stream_name: %s\n", smb_fname_str_dbg(stream_name));
 
+	rc = vfs_stat(handle->conn, stream_name);
+	if (rc == -1 && errno != ENOENT) {
+		return false;
+	}
+
+	status = openat_pathref_fsp(handle->conn->cwd_fsp, stream_name);
+	if (!NT_STATUS_IS_OK(status) &&
+	    !NT_STATUS_EQUAL(status, NT_STATUS_OBJECT_NAME_NOT_FOUND))
+	{
+		return false;
+	}
+
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,			/* conn */
 		NULL,				/* req */
-		&handle->conn->cwd_fsp,		/* dirfsp */
 		stream_name,			/* fname */
 		FILE_GENERIC_WRITE,		/* access_mask */
 		FILE_SHARE_READ | FILE_SHARE_WRITE, /* share_access */
@@ -1370,12 +1395,12 @@ static bool ad_convert_finderinfo(vfs_handle_struct *handle,
 	if (nwritten == -1) {
 		DBG_ERR("SMB_VFS_PWRITE failed\n");
 		saved_errno = errno;
-		close_file(NULL, fsp, ERROR_CLOSE);
+		close_file_free(NULL, &fsp, ERROR_CLOSE);
 		errno = saved_errno;
 		return false;
 	}
 
-	status = close_file(NULL, fsp, NORMAL_CLOSE);
+	status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 	if (!NT_STATUS_IS_OK(status)) {
 		return false;
 	}
@@ -1448,11 +1473,13 @@ static bool ad_convert_blank_rfork(vfs_handle_struct *handle,
 
 static bool ad_convert_delete_adfile(vfs_handle_struct *handle,
 				struct adouble *ad,
-				struct files_struct *dirfsp,
 				const struct smb_filename *smb_fname,
 				uint32_t flags)
 {
+	struct smb_filename *parent_fname = NULL;
+	struct smb_filename *at_fname = NULL;
 	struct smb_filename *ad_name = NULL;
+	NTSTATUS status;
 	int rc;
 
 	if (ad_getentrylen(ad, ADEID_RFORK) > 0) {
@@ -1468,19 +1495,32 @@ static bool ad_convert_delete_adfile(vfs_handle_struct *handle,
 		return false;
 	}
 
-	rc = SMB_VFS_NEXT_UNLINKAT(handle,
-			dirfsp,
-			ad_name,
-			0);
-	if (rc != 0) {
-		DBG_ERR("Unlinking [%s] failed: %s\n",
-			smb_fname_str_dbg(ad_name), strerror(errno));
-		TALLOC_FREE(ad_name);
+	status = parent_pathref(talloc_tos(),
+				handle->conn->cwd_fsp,
+				ad_name,
+				&parent_fname,
+				&at_fname);
+	TALLOC_FREE(ad_name);
+	if (!NT_STATUS_IS_OK(status)) {
 		return false;
 	}
 
-	DBG_WARNING("Unlinked [%s] after conversion\n", smb_fname_str_dbg(ad_name));
-	TALLOC_FREE(ad_name);
+	rc = SMB_VFS_NEXT_UNLINKAT(handle,
+			parent_fname->fsp,
+			at_fname,
+			0);
+	if (rc != 0) {
+		DBG_ERR("Unlinking [%s/%s] failed: %s\n",
+			smb_fname_str_dbg(parent_fname),
+			smb_fname_str_dbg(at_fname), strerror(errno));
+		TALLOC_FREE(parent_fname);
+		return false;
+	}
+
+	DBG_WARNING("Unlinked [%s/%s] after conversion\n",
+		    smb_fname_str_dbg(parent_fname),
+		    smb_fname_str_dbg(at_fname));
+	TALLOC_FREE(parent_fname);
 
 	return true;
 }
@@ -1495,7 +1535,6 @@ static bool ad_convert_delete_adfile(vfs_handle_struct *handle,
  * otherwise
  **/
 int ad_convert(struct vfs_handle_struct *handle,
-		struct files_struct *dirfsp,
 		const struct smb_filename *smb_fname,
 		const char *catia_mappings,
 		uint32_t flags)
@@ -1545,7 +1584,6 @@ int ad_convert(struct vfs_handle_struct *handle,
 
 	ok = ad_convert_delete_adfile(handle,
 			ad,
-			dirfsp,
 			smb_fname,
 			flags);
 	if (!ok) {
@@ -1569,10 +1607,21 @@ static bool ad_unconvert_open_ad(TALLOC_CTX *mem_ctx,
 	NTSTATUS status;
 	int ret;
 
+	ret = vfs_stat(handle->conn, adpath);
+	if (ret == -1 && errno != ENOENT) {
+		return false;
+	}
+
+	if (VALID_STAT(adpath->st)) {
+		status = openat_pathref_fsp(handle->conn->cwd_fsp, adpath);
+		if (!NT_STATUS_IS_OK(status)) {
+			return false;
+		}
+	}
+
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,
 		NULL,				/* req */
-		&handle->conn->cwd_fsp,		/* dirfsp */
 		adpath,
 		FILE_READ_DATA|FILE_WRITE_DATA,
 		FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
@@ -1603,7 +1652,7 @@ static bool ad_unconvert_open_ad(TALLOC_CTX *mem_ctx,
 		if (ret != 0) {
 			DBG_ERR("SMB_VFS_FCHOWN [%s] failed: %s\n",
 				fsp_str_dbg(fsp), nt_errstr(status));
-			close_file(NULL, fsp, NORMAL_CLOSE);
+			close_file_free(NULL, &fsp, NORMAL_CLOSE);
 			return false;
 		}
 	}
@@ -1621,10 +1670,18 @@ static bool ad_unconvert_get_streams(struct vfs_handle_struct *handle,
 	files_struct *fsp = NULL;
 	NTSTATUS status;
 
+	if (!VALID_STAT(smb_fname->st)) {
+		return false;
+	}
+
+	status = openat_pathref_fsp(handle->conn->cwd_fsp, smb_fname);
+	if (!NT_STATUS_IS_OK(status)) {
+		return false;
+	}
+
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,				/* conn */
 		NULL,					/* req */
-		&handle->conn->cwd_fsp,			/* dirfsp */
 		smb_fname,				/* fname */
 		FILE_READ_ATTRIBUTES,			/* access_mask */
 		(FILE_SHARE_READ | FILE_SHARE_WRITE |	/* share_access */
@@ -1648,21 +1705,19 @@ static bool ad_unconvert_get_streams(struct vfs_handle_struct *handle,
 		return false;
 	}
 
-	status = vfs_streaminfo(handle->conn,
-				fsp,
-				fsp->fsp_name,
+	status = vfs_fstreaminfo(fsp,
 				mem_ctx,
 				num_streams,
 				streams);
 	if (!NT_STATUS_IS_OK(status)) {
-		close_file(NULL, fsp, NORMAL_CLOSE);
+		close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		DBG_ERR("streaminfo on [%s] failed: %s\n",
 			smb_fname_str_dbg(smb_fname),
 			nt_errstr(status));
 		return false;
 	}
 
-	status = close_file(NULL, fsp, NORMAL_CLOSE);
+	status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("close_file [%s] failed: %s\n",
 			smb_fname_str_dbg(smb_fname),
@@ -1721,10 +1776,15 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 		goto out;
 	}
 
+	status = openat_pathref_fsp(handle->conn->cwd_fsp, sname);
+	if (!NT_STATUS_IS_OK(status)) {
+		ok = false;
+		goto out;
+	}
+
 	status = SMB_VFS_CREATE_FILE(
 		handle->conn,
 		NULL,				/* req */
-		&handle->conn->cwd_fsp,		/* dirfsp */
 		sname,
 		FILE_READ_DATA|DELETE_ACCESS,
 		FILE_SHARE_READ,
@@ -1915,7 +1975,7 @@ static bool ad_collect_one_stream(struct vfs_handle_struct *handle,
 out:
 	TALLOC_FREE(sname);
 	if (fsp != NULL) {
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
 			DBG_ERR("close_file [%s] failed: %s\n",
 				smb_fname_str_dbg(smb_fname),
@@ -2057,9 +2117,9 @@ bool ad_unconvert(TALLOC_CTX *mem_ctx,
 
 out:
 	if (fsp != NULL) {
-		status = close_file(NULL, fsp, NORMAL_CLOSE);
+		status = close_file_free(NULL, &fsp, NORMAL_CLOSE);
 		if (!NT_STATUS_IS_OK(status)) {
-			DBG_ERR("close_file [%s] failed: %s\n",
+			DBG_ERR("close_file_free() [%s] failed: %s\n",
 				smb_fname_str_dbg(smb_fname),
 				nt_errstr(status));
 			ok = false;
@@ -2079,12 +2139,19 @@ static ssize_t ad_read_meta(vfs_handle_struct *handle,
 	int      rc = 0;
 	ssize_t  ealen;
 	bool     ok;
+	struct files_struct *fsp = smb_fname->fsp;
 
 	DEBUG(10, ("reading meta xattr for %s\n", smb_fname->base_name));
 
-	ealen = SMB_VFS_GETXATTR(handle->conn, smb_fname,
-				 AFPINFO_EA_NETATALK, ad->ad_data,
-				 AD_DATASZ_XATTR);
+	if (fsp->base_fsp != NULL) {
+		fsp = fsp->base_fsp;
+	}
+
+	ealen = SMB_VFS_FGETXATTR(fsp,
+				  AFPINFO_EA_NETATALK,
+				  ad->ad_data,
+				  AD_DATASZ_XATTR);
+
 	if (ealen == -1) {
 		switch (errno) {
 		case ENOATTR:
@@ -2139,9 +2206,8 @@ exit:
 		ealen = -1;
 		if (errno == EINVAL) {
 			become_root();
-			(void)SMB_VFS_REMOVEXATTR(handle->conn,
-						  smb_fname,
-						  AFPINFO_EA_NETATALK);
+			(void)SMB_VFS_FREMOVEXATTR(fsp,
+						   AFPINFO_EA_NETATALK);
 			unbecome_root();
 			errno = ENOENT;
 		}
@@ -2149,67 +2215,95 @@ exit:
 	return ealen;
 }
 
-static int ad_open_rsrc(vfs_handle_struct *handle,
-			const struct smb_filename *smb_fname,
-			int flags,
-			mode_t mode,
-			files_struct **_fsp)
+static NTSTATUS adouble_open_rsrc_fsp(const struct files_struct *dirfsp,
+				      const struct smb_filename *smb_base_fname,
+				      int in_flags,
+				      mode_t mode,
+				      struct files_struct **_ad_fsp)
 {
-	int ret;
+	int rc = 0;
+	struct adouble *ad = NULL;
 	struct smb_filename *adp_smb_fname = NULL;
-	files_struct *fsp = NULL;
-	uint32_t access_mask;
-	uint32_t share_access;
-	uint32_t create_disposition;
+	struct files_struct *ad_fsp = NULL;
 	NTSTATUS status;
+	int flags = in_flags;
 
-	ret = adouble_path(talloc_tos(), smb_fname, &adp_smb_fname);
-	if (ret != 0) {
-		return -1;
+	rc = adouble_path(talloc_tos(),
+			  smb_base_fname,
+			  &adp_smb_fname);
+	if (rc != 0) {
+		return NT_STATUS_NO_MEMORY;
 	}
 
-	ret = SMB_VFS_STAT(handle->conn, adp_smb_fname);
-	if (ret != 0) {
-		TALLOC_FREE(adp_smb_fname);
-		return -1;
-	}
-
-	access_mask = FILE_GENERIC_READ;
-	share_access = FILE_SHARE_READ | FILE_SHARE_WRITE;
-	create_disposition = FILE_OPEN;
-
-	if (flags & O_RDWR) {
-		access_mask |= FILE_GENERIC_WRITE;
-		share_access &= ~FILE_SHARE_WRITE;
-	}
-
-	status = SMB_VFS_CREATE_FILE(
-		handle->conn,			/* conn */
-		NULL,				/* req */
-		&handle->conn->cwd_fsp,		/* dirfsp */
-		adp_smb_fname,
-		access_mask,
-		share_access,
-		create_disposition,
-		0,				/* create_options */
-		0,				/* file_attributes */
-		INTERNAL_OPEN_ONLY,		/* oplock_request */
-		NULL,				/* lease */
-		0,				/* allocation_size */
-		0,				/* private_flags */
-		NULL,				/* sd */
-		NULL,				/* ea_list */
-		&fsp,
-		NULL,				/* psbuf */
-		NULL, NULL);			/* create context */
-	TALLOC_FREE(adp_smb_fname);
+	status = create_internal_fsp(dirfsp->conn,
+				     adp_smb_fname,
+				     &ad_fsp);
 	if (!NT_STATUS_IS_OK(status)) {
-		DBG_ERR("SMB_VFS_CREATE_FILE failed\n");
-		return -1;
+		return status;
 	}
 
-	*_fsp = fsp;
-	return 0;
+#ifdef O_PATH
+	flags &= ~(O_PATH);
+#endif
+	if (flags & (O_CREAT | O_TRUNC | O_WRONLY)) {
+		/* We always need read/write access for the metadata header too */
+		flags &= ~(O_WRONLY);
+		flags |= O_RDWR;
+	}
+
+	status = fd_openat(dirfsp,
+			   adp_smb_fname,
+			   ad_fsp,
+			   flags,
+			   mode);
+	if (!NT_STATUS_IS_OK(status)) {
+		file_free(NULL, ad_fsp);
+		return status;
+	}
+
+	if (flags & (O_CREAT | O_TRUNC)) {
+		ad = ad_init(talloc_tos(), ADOUBLE_RSRC);
+		if (ad == NULL) {
+			file_free(NULL, ad_fsp);
+			return NT_STATUS_NO_MEMORY;
+		}
+
+		rc = ad_fset(ad_fsp->conn->vfs_handles, ad, ad_fsp);
+		if (rc != 0) {
+			file_free(NULL, ad_fsp);
+			return NT_STATUS_IO_DEVICE_ERROR;
+		}
+		TALLOC_FREE(ad);
+	}
+
+	*_ad_fsp = ad_fsp;
+	return NT_STATUS_OK;
+}
+
+NTSTATUS adouble_open_from_base_fsp(const struct files_struct *dirfsp,
+				    struct files_struct *base_fsp,
+				    adouble_type_t type,
+				    int flags,
+				    mode_t mode,
+				    struct files_struct **_ad_fsp)
+{
+	*_ad_fsp = NULL;
+
+	SMB_ASSERT(base_fsp != NULL);
+	SMB_ASSERT(base_fsp->base_fsp == NULL);
+
+	switch (type) {
+	case ADOUBLE_META:
+		return NT_STATUS_INTERNAL_ERROR;
+	case ADOUBLE_RSRC:
+		return adouble_open_rsrc_fsp(dirfsp,
+					     base_fsp->fsp_name,
+					     flags,
+					     mode,
+					     _ad_fsp);
+	}
+
+	return NT_STATUS_INTERNAL_ERROR;
 }
 
 /*
@@ -2224,7 +2318,7 @@ static int ad_open(vfs_handle_struct *handle,
 		   int flags,
 		   mode_t mode)
 {
-	int ret;
+	NTSTATUS status;
 
 	DBG_DEBUG("Path [%s] type [%s]\n", smb_fname->base_name,
 		  ad->ad_type == ADOUBLE_META ? "meta" : "rsrc");
@@ -2239,8 +2333,13 @@ static int ad_open(vfs_handle_struct *handle,
 		return 0;
 	}
 
-	ret = ad_open_rsrc(handle, smb_fname, flags, mode, &ad->ad_fsp);
-	if (ret != 0) {
+	status = adouble_open_rsrc_fsp(handle->conn->cwd_fsp,
+				       smb_fname,
+				       flags,
+				       mode,
+				       &ad->ad_fsp);
+	if (!NT_STATUS_IS_OK(status)) {
+		errno = map_errno_from_nt_status(status);
 		return -1;
 	}
 	ad->ad_opened = true;
@@ -2345,11 +2444,14 @@ static int adouble_destructor(struct adouble *ad)
 
 	SMB_ASSERT(ad->ad_fsp != NULL);
 
-	status = close_file(NULL, ad->ad_fsp, NORMAL_CLOSE);
+	status = fd_close(ad->ad_fsp);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("Closing [%s] failed: %s\n",
 			fsp_str_dbg(ad->ad_fsp), nt_errstr(status));
 	}
+	file_free(NULL, ad->ad_fsp);
+	ad->ad_fsp = NULL;
+	ad->ad_opened = false;
 
 	return 0;
 }
@@ -2488,7 +2590,11 @@ static struct adouble *ad_get_internal(TALLOC_CTX *ctx,
 	int mode;
 
 	if (fsp != NULL) {
-		smb_fname = fsp->base_fsp->fsp_name;
+		if (fsp->base_fsp != NULL) {
+			smb_fname = fsp->base_fsp->fsp_name;
+		} else {
+			smb_fname = fsp->fsp_name;
+		}
 	}
 
 	DEBUG(10, ("ad_get(%s) called for %s\n",
@@ -2569,46 +2675,6 @@ struct adouble *ad_fget(TALLOC_CTX *ctx, vfs_handle_struct *handle,
 	return ad_get_internal(ctx, handle, fsp, NULL, type);
 }
 
-/**
- * Set AppleDouble metadata on a file or directory
- *
- * @param[in] ad      adouble handle
- *
- * @param[in] smb_fname    pathname to file or directory
- *
- * @return            status code, 0 means success
- **/
-int ad_set(vfs_handle_struct *handle,
-	   struct adouble *ad,
-	   const struct smb_filename *smb_fname)
-{
-	bool ok;
-	int ret;
-
-	DBG_DEBUG("Path [%s]\n", smb_fname->base_name);
-
-	if (ad->ad_type != ADOUBLE_META) {
-		DBG_ERR("ad_set on [%s] used with ADOUBLE_RSRC\n",
-			smb_fname->base_name);
-		return -1;
-	}
-
-	ok = ad_pack(handle, ad, NULL);
-	if (!ok) {
-		return -1;
-	}
-
-	ret = SMB_VFS_SETXATTR(handle->conn,
-			       smb_fname,
-			       AFPINFO_EA_NETATALK,
-			       ad->ad_data,
-			       AD_DATASZ_XATTR, 0);
-
-	DBG_DEBUG("Path [%s] ret [%d]\n", smb_fname->base_name, ret);
-
-	return ret;
-}
-
 /**
  * Set AppleDouble metadata on a file or directory
  *
@@ -2627,13 +2693,6 @@ int ad_fset(struct vfs_handle_struct *handle,
 
 	DBG_DEBUG("Path [%s]\n", fsp_str_dbg(fsp));
 
-	if ((fsp == NULL)
-	    || (fsp->fh == NULL)
-	    || (fsp->fh->fd == -1))
-	{
-		smb_panic("bad fsp");
-	}
-
 	ok = ad_pack(handle, ad, fsp);
 	if (!ok) {
 		return -1;
@@ -2641,13 +2700,12 @@ int ad_fset(struct vfs_handle_struct *handle,
 
 	switch (ad->ad_type) {
 	case ADOUBLE_META:
-		rc = SMB_VFS_NEXT_SETXATTR(handle,
-					   fsp->fsp_name,
-					   AFPINFO_EA_NETATALK,
-					   ad->ad_data,
-					   AD_DATASZ_XATTR, 0);
+		rc = SMB_VFS_NEXT_FSETXATTR(handle,
+				   fsp->base_fsp ? fsp->base_fsp : fsp,
+				   AFPINFO_EA_NETATALK,
+				   ad->ad_data,
+				   AD_DATASZ_XATTR, 0);
 		break;
-
 	case ADOUBLE_RSRC:
 		len = SMB_VFS_NEXT_PWRITE(handle,
 					  fsp,
@@ -2722,8 +2780,13 @@ int adouble_path(TALLOC_CTX *ctx,
 		return -1;
 	}
 
-	smb_fname->base_name = talloc_asprintf(smb_fname,
+	if (ISDOT(parent)) {
+		smb_fname->base_name = talloc_asprintf(smb_fname,
+					"._%s", base);
+	} else {
+		smb_fname->base_name = talloc_asprintf(smb_fname,
 					"%s/._%s", parent, base);
+	}
 	if (smb_fname->base_name == NULL) {
 		TALLOC_FREE(smb_fname);
 		return -1;
