@@ -26,8 +26,9 @@
 #include "system/kerberos.h"
 #include "auth/kerberos/kerberos.h"
 #include <hdb.h>
+#include <heimbase.h>
 #include <kdc.h>
-#include <krb5/windc_plugin.h>
+#include <krb5/kdc-plugin.h>
 #include "kdc/samba_kdc.h"
 #include "kdc/kdc-server.h"
 
@@ -52,5 +53,5 @@ NTSTATUS hdb_samba4_kpasswd_create_kdc(struct samba_kdc_base_context *base_ctx,
 int kdc_check_pac(krb5_context krb5_context,
 		  DATA_BLOB server_sig,
 		  struct PAC_SIGNATURE_DATA *kdc_sig,
-		  hdb_entry_ex *ent);
+		  hdb_entry *ent);
 #endif
