@@ -116,7 +116,7 @@ static bool test_find(struct torture_context *tctx,
 	NTSTATUS status;
 	bool ret = true;
 	unsigned int count;
-	int i, j, file_count = 0;
+	int i, j = 0, file_count = 0;
 
 	status = populate_tree(tctx, mem_ctx, tree, files, NFILES, &h);
 
@@ -312,35 +312,59 @@ static struct {
 	NTSTATUS status;
 	union smb_search_data data;
 } levels[] = {
-	{"SMB2_FIND_DIRECTORY_INFO",
-	 SMB2_FIND_DIRECTORY_INFO, RAW_SEARCH_DATA_DIRECTORY_INFO,
-	 offsetof(union smb_search_data, directory_info.name.s),
-	 offsetof(union smb_search_data, directory_info.file_index),
+	{
+		.name              = "SMB2_FIND_DIRECTORY_INFO",
+		.level             = SMB2_FIND_DIRECTORY_INFO,
+		.data_level        = RAW_SEARCH_DATA_DIRECTORY_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      directory_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+					      directory_info.file_index),
 	},
-	{"SMB2_FIND_FULL_DIRECTORY_INFO",
-	 SMB2_FIND_FULL_DIRECTORY_INFO, RAW_SEARCH_DATA_FULL_DIRECTORY_INFO,
-	 offsetof(union smb_search_data, full_directory_info.name.s),
-	 offsetof(union smb_search_data, full_directory_info.file_index),
+	{
+		.name              = "SMB2_FIND_FULL_DIRECTORY_INFO",
+		.level             = SMB2_FIND_FULL_DIRECTORY_INFO,
+		.data_level        = RAW_SEARCH_DATA_FULL_DIRECTORY_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      full_directory_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+					      full_directory_info.file_index),
 	},
-	{"SMB2_FIND_NAME_INFO",
-	 SMB2_FIND_NAME_INFO, RAW_SEARCH_DATA_NAME_INFO,
-	 offsetof(union smb_search_data, name_info.name.s),
-	 offsetof(union smb_search_data, name_info.file_index),
+	{
+		.name              = "SMB2_FIND_NAME_INFO",
+		.level             = SMB2_FIND_NAME_INFO,
+		.data_level        = RAW_SEARCH_DATA_NAME_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      name_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+					      name_info.file_index),
 	},
-	{"SMB2_FIND_BOTH_DIRECTORY_INFO",
-	 SMB2_FIND_BOTH_DIRECTORY_INFO, RAW_SEARCH_DATA_BOTH_DIRECTORY_INFO,
-	 offsetof(union smb_search_data, both_directory_info.name.s),
-	 offsetof(union smb_search_data, both_directory_info.file_index),
+	{
+		.name              = "SMB2_FIND_BOTH_DIRECTORY_INFO",
+		.level             = SMB2_FIND_BOTH_DIRECTORY_INFO,
+		.data_level        = RAW_SEARCH_DATA_BOTH_DIRECTORY_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      both_directory_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+					      both_directory_info.file_index),
 	},
-	{"SMB2_FIND_ID_FULL_DIRECTORY_INFO",
-	 SMB2_FIND_ID_FULL_DIRECTORY_INFO, RAW_SEARCH_DATA_ID_FULL_DIRECTORY_INFO,
-	 offsetof(union smb_search_data, id_full_directory_info.name.s),
-	 offsetof(union smb_search_data, id_full_directory_info.file_index),
+	{
+		.name              = "SMB2_FIND_ID_FULL_DIRECTORY_INFO",
+		.level             = SMB2_FIND_ID_FULL_DIRECTORY_INFO,
+		.data_level        = RAW_SEARCH_DATA_ID_FULL_DIRECTORY_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      id_full_directory_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+				              id_full_directory_info.file_index),
 	},
-	{"SMB2_FIND_ID_BOTH_DIRECTORY_INFO",
-	 SMB2_FIND_ID_BOTH_DIRECTORY_INFO, RAW_SEARCH_DATA_ID_BOTH_DIRECTORY_INFO,
-	 offsetof(union smb_search_data, id_both_directory_info.name.s),
-	 offsetof(union smb_search_data, id_both_directory_info.file_index),
+	{
+		.name              = "SMB2_FIND_ID_BOTH_DIRECTORY_INFO",
+		.level             = SMB2_FIND_ID_BOTH_DIRECTORY_INFO,
+		.data_level        = RAW_SEARCH_DATA_ID_BOTH_DIRECTORY_INFO,
+		.name_offset       = offsetof(union smb_search_data,
+					      id_both_directory_info.name.s),
+		.resume_key_offset = offsetof(union smb_search_data,
+					      id_both_directory_info.file_index),
 	}
 };
 
@@ -1276,7 +1300,7 @@ static bool test_large_files(struct torture_context *tctx,
 	struct smb2_find f;
 	struct smb2_handle h = {{0}};
 	union smb_search_data *d;
-	int i, j, file_count = 0;
+	int i, j = 0, file_count = 0;
 	char **strs = NULL;
 	unsigned count;
 
